==> fap-0.9.0pre1.tar.bz2: fap-0.9.0pre1/src/audio/input/sidBackend.h <==
/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SID_H
#define SID_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#if defined (HAVE_SIDPLAY_BUILDERS_RESID_H) || defined (HAVE_SIDPLAY_BUILDERS_HARDSID_H) || defined (HAVE_SIDPLAYFP)

#ifdef HAVE_SIDPLAYFP
#  include <sidplayfp/sidplay2.h>
#else
#  include <sidplay/sidplay2.h>
#endif

#define HAVE_SIDPLAY2

#ifdef HAVE_SIDPLAYFP
# include <sidplayfp/SidDatabase.h>
#elif defined (HAVE_SIDUTILS)
# include <sidplay/utils/SidDatabase.h>
#endif


#include <fx.h>
#include "inputBackend.h"
#include "stilview/stil.h"

/*****************************************************************/

typedef struct {
	FXString	engine;
	FXint		samplerate;
	FXint		channels;
#ifndef HAVE_SIDPLAYFP
	sample_t	precision;
	FXint		optimisation;
#else
	sampling_method_t	samplingMethod;
	FXbool		fastSampling;
#endif
	sid2_env_t	environment;
	sid2_clock_t	clock;
	FXbool		forceClock;
	sid2_model_t	model;
	FXbool		filter;
	FXString	hvscPath;
	FXbool		psid_filter;
#if defined HAVE_SIDUTILS || defined (HAVE_SIDPLAYFP)
	FXString	filterIni;
#endif
} sidConfig_t;

/*****************************************************************/

#include "configFrame.h"

class sidConfig : public configFrame {
	FXDECLARE(sidConfig)

private:
#ifndef HAVE_SIDPLAYFP
	FXDataTarget		_optimisation;
#else
	FXDataTarget		_fastSampling;
#endif
	FXDataTarget		_filter;
	FXDataTarget		_hvscDir;
#if defined HAVE_SIDUTILS || defined (HAVE_SIDPLAYFP)
	FXDataTarget		_filterIni;
#endif

private:
	sidConfig() {};
	sidConfig(const sidConfig&);
	sidConfig& operator=(const sidConfig&);

public:
	sidConfig(FXPacker* win);
	virtual ~sidConfig() {};

public:
	enum{
		ID_FREQUENCY=FXDialogBox::ID_LAST,
		ID_CHANNELS,
#ifndef HAVE_SIDPLAYFP
		ID_BITS,
#endif
		ID_ENGINE,
		ID_ENVIRONMENT,
		ID_HVSC,
		ID_FILTER,
		ID_SAMPLING,
		ID_LAST,
	};

public:
	long onCmdFrequency(FXObject*, FXSelector, void*);
	long onCmdChannels(FXObject*, FXSelector, void*);
#ifndef HAVE_SIDPLAYFP
	long onCmdBits(FXObject*, FXSelector, void*);
#else
	long onCmdSampling(FXObject*, FXSelector, void*);
#endif
	long onCmdEngine(FXObject*, FXSelector, void*);
	long onCmdEnvironment(FXObject*, FXSelector, void*);
	long onCmdHvsc(FXObject*, FXSelector, void*);
	long onCmdFilter(FXObject*, FXSelector, void*);
};

/*****************************************************************/

class sidBackend : public inputBackend {

	friend class sidConfig;

private:
	static sidConfig_t	_settings;
	static const char	_engines[3][8];

private:
	sidplay2	*_sidplay2;

	STIL		*_stil;

#if defined HAVE_SIDUTILS || defined (HAVE_SIDPLAYFP)
	SidDatabase	*_db;
	FXuint		_length;
	char		_md5[SIDTUNE_MD5_LENGTH+1];

	SidTuneMod	*_tune;
#else
	SidTune		*_tune;
#endif

private:
	void loadTune(const int num);

	void openHvsc(const FXString& hvscPath);

public:
	sidBackend();
	~sidBackend();

	static const char name[];

	/// Factory method
	static input* factory() { return new sidBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Number of subtunes
	unsigned int subtunes() const { return _tune?_tune->getInfo().songs:0; }

	/// Current subtune
	unsigned int subtune() const { return _tune?_tune->getInfo().currentSong:0; }

	/// Change subtune
	bool subtune(const unsigned int i);

	/// Get samplerate
	const unsigned int samplerate() const { return _tune?_settings.samplerate:0; }

	/// Get channels
	const unsigned int channels() const { return _tune?_settings.channels:0; }

	/// Get precision
	const sample_t precision() const {
#ifndef HAVE_SIDPLAYFP
		return _settings.precision;
#else
		return S16;
#endif
	}

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Open config dialog
	void config(FXPacker* win) { new sidConfig(win); }

	/// Get Music directory
	const FXString getMusicDir(void) { return _settings.hvscPath; }

	void loadSettings();

	void saveSettings();
};

#endif

#endif

==> fap-0.9.2.tar.bz2: fap-0.9.2/src/audio/input/sidBackend.h <==
/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SID_H
#define SID_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#if defined (HAVE_SIDPLAY_BUILDERS_RESID_H) || defined (HAVE_SIDPLAY_BUILDERS_HARDSID_H) || defined (HAVE_SIDPLAYFP)

#ifdef HAVE_SIDPLAYFP
#  include <sidplayfp/sidplay2.h>
#else
#  include <sidplay/sidplay2.h>
#endif

#define HAVE_SIDPLAY2

#ifdef HAVE_SIDPLAYFP
# include <sidplayfp/SidDatabase.h>
# include <sidplayfp/SidTuneMod.h>
#elif defined (HAVE_SIDUTILS)
# include <sidplay/utils/SidDatabase.h>
#endif


#include <fx.h>
#include "inputBackend.h"
#include "stilview/stil.h"

/*****************************************************************/

typedef struct {
	FXString	engine;
	FXint		samplerate;
	FXint		channels;
#ifndef HAVE_SIDPLAYFP
	sample_t	precision;
	FXint		optimisation;
#else
	sampling_method_t	samplingMethod;
	FXbool		fastSampling;
	FXint		bias;
	FXint		filter6581Curve;
	FXint		filter8580Curve;
#endif
	sid2_env_t	environment;
	sid2_clock_t	clock;
	FXbool		forceClock;
	sid2_model_t	model;
	FXbool		filter;
	FXString	hvscPath;
	FXbool		psid_filter;
#if defined HAVE_SIDUTILS
	FXString	filterIni;
#endif
} sidConfig_t;

/*****************************************************************/

#include "configFrame.h"

class sidConfig : public configFrame {
	FXDECLARE(sidConfig)

private:
#ifndef HAVE_SIDPLAYFP
	FXDataTarget		_optimisation;
#else
	FXDataTarget		_fastSampling;
	FXDataTarget		_bias;
	FXDataTarget		_filter6581Curve;
	FXDataTarget		_filter8580Curve;

	FXVerticalFrame*	_biasFrame;
	FXVerticalFrame*	_filterCurveFrame;
#endif
	FXDataTarget		_forceClock;
	FXDataTarget		_filter;
	FXDataTarget		_hvscDir;
#if defined HAVE_SIDUTILS
	FXDataTarget		_filterIni;
#endif

private:
	sidConfig() {};
	sidConfig(const sidConfig&);
	sidConfig& operator=(const sidConfig&);

public:
	sidConfig(FXPacker* win);
	virtual ~sidConfig() {};

public:
	enum{
		ID_FREQUENCY=FXDialogBox::ID_LAST,
		ID_CHANNELS,
#ifndef HAVE_SIDPLAYFP
		ID_BITS,
#endif
		ID_ENGINE,
		ID_ENVIRONMENT,
		ID_CLOCK,
		ID_MODEL,
		ID_HVSC,
#if defined HAVE_SIDUTILS
		ID_FILTER,
#endif
		ID_SAMPLING,
		ID_LAST,
	};

public:
	long onCmdFrequency(FXObject*, FXSelector, void*);
	long onCmdChannels(FXObject*, FXSelector, void*);
#ifndef HAVE_SIDPLAYFP
	long onCmdBits(FXObject*, FXSelector, void*);
#else
	long onCmdSampling(FXObject*, FXSelector, void*);
#endif
	long onCmdEngine(FXObject*, FXSelector, void*);
	long onCmdEnvironment(FXObject*, FXSelector, void*);
	long onCmdClock(FXObject*, FXSelector, void*);
	long onCmdModel(FXObject*, FXSelector, void*);
	long onCmdHvsc(FXObject*, FXSelector, void*);
#if defined HAVE_SIDUTILS
	long onCmdFilter(FXObject*, FXSelector, void*);
#endif
};

/*****************************************************************/

class sidBackend : public inputBackend {

	friend class sidConfig;

private:
	static sidConfig_t	_settings;
	static const char	_engines[3][8];

private:
	sidplay2	*_sidplay2;

	STIL		*_stil;

#if defined HAVE_SIDUTILS || defined (HAVE_SIDPLAYFP)
	SidDatabase	*_db;
	FXuint		_length;
	char		_md5[SIDTUNE_MD5_LENGTH+1];

	SidTuneMod	*_tune;
#else
	SidTune		*_tune;
#endif

private:
	sidBackend();

	void loadTune(const int num);

	void openHvsc(const FXString& hvscPath);

public:
	~sidBackend();

	static const char name[];

	/// Factory method
	static input* factory() { return new sidBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	bool rewind();

	/// Number of subtunes
	unsigned int subtunes() const { return _tune?_tune->getInfo().songs:0; }

	/// Current subtune
	unsigned int subtune() const { return _tune?_tune->getInfo().currentSong:0; }

	/// Change subtune
	bool subtune(const unsigned int i);

	/// Get samplerate
	unsigned int samplerate() const { return _tune?_settings.samplerate:0; }

	/// Get channels
	unsigned int channels() const { return _tune?_settings.channels:0; }

	/// Get precision
	sample_t precision() const {
#ifndef HAVE_SIDPLAYFP
		return _settings.precision;
#else
		return S16;
#endif
	}

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Open config dialog
	void config(FXPacker* win) { new sidConfig(win); }

	/// Get Music directory
	const FXString getMusicDir(void) { return _settings.hvscPath; }

	void loadSettings();

	void saveSettings();
};

#endif

#endif
